import logging
import os
import sys

from nose.plugins import Plugin

log = logging.getLogger('nose.plugins.ferrisnose')


class FerrisNose(Plugin):
    name = 'ferris'

    def options(self, parser, env=os.environ):
        super(FerrisNose, self).options(parser, env=env)

        parser.add_option(
            '--gae-sdk-path', default=env.get('APPENGINE_SDK_PATH', os.path.expanduser('~/google-cloud-sdk/platform/google_appengine')),
            dest='gae_sdk_path',
            help='Set the path to the directory of the App Engine SDK installation (you can also use the APPENGINE_SDK_PATH environment variable)')

    def configure(self, options, conf):
        super(FerrisNose, self).configure(options, conf)

        if not self.enabled:
            return

        if not conf.testNames:
            log.warning('No test paths specified assuming app/tests')
            conf.testNames.append('app/tests')

        self.gae_path = options.gae_sdk_path

        self._check_path()
        self._setup_path()
        self._setup_testbed()
        self._setup_gae_config()
        self._setup_logging()

    def _check_path(self):
        wd = os.getcwd()
        if not os.path.exists(os.path.join(wd, 'app.yaml')):
            log.warning("No app.yaml found, this could lead to problems.")

    def _setup_path(self):
        # Load the app engine path into sys
        sys.path.append(self.gae_path)

        # store the current path
        current_path = sys.path[:]

        # Try to import the app server
        try:
            import dev_appserver
        except ImportError:
            raise ValueError("Could not locate the App Engine SDK. Please provide the SDK path using the --gae-sdk-path argument or set the APPENGINE_SDK_PATH environment variable.")

        # make appengine load its libraries
        dev_appserver.fix_sys_path()

        # Reload the google module
        if 'google' in sys.modules:
            import google
            reload(google)

        # Restore the path and add the current directory to the path
        sys.path.extend(current_path)
        sys.path.append(os.getcwd())

    def _setup_testbed(self):
        # Activate a testbed during test discovery
        from .testbed import SimpleTestBed
        self.testbed = SimpleTestBed()
        self.testbed.activate()

    def _setup_gae_config(self):
        # Import appengine config
        try:
            import appengine_config
            logging.info("appengine_config.py loaded")
        except:
            logging.info("Failed to load appengine_config.py")
            pass

    def _setup_logging(self):
        # Remove agressive logging
        rootLogger = logging.getLogger()
        rootLogger.setLevel(logging.INFO)
        for handler in rootLogger.handlers:
            if isinstance(handler, logging.StreamHandler):
                rootLogger.removeHandler(handler)

    def beforeTest(self, test):
        # Turn off the internal testbed before running a test.
        self.testbed.deactivate()



from google.appengine.ext import testbed
from google.appengine.api.blobstore import blobstore_stub, file_blob_storage
from google.appengine.api.files import file_service_stub
from google.appengine.datastore import datastore_stub_util
from google.appengine.api.search.simple_search_stub import SearchServiceStub
from google.appengine.ext import deferred
import base64
import tempfile
import os
import logging


class SimpleTestBed(object):
    """
    Creates a limited testbed, this is used before loading test
    files to fix issues with modules that touch google services before
    an http request. This also fixes the httplib2 library from being
    obtuse during tests.
    """
    def __init__(self):
        self.testbed = testbed.Testbed()

    def activate(self):
        self.testbed.activate()
        policy = datastore_stub_util.PseudoRandomHRConsistencyPolicy(probability=1)
        self.testbed.init_datastore_v3_stub(consistency_policy=policy)
        self.testbed.init_urlfetch_stub()
        self.testbed.init_memcache_stub()
        self.testbed.init_logservice_stub()

    def deactivate(self):
        try:
            self.testbed.deactivate()
        except:
            pass


class FullTestBed(object):
    """
    Creates a full testbed for use during tests.
    """
    def __init__(self):
        self.testbed = testbed.Testbed()

    def activate(self):
        self.testbed.setup_env(
            AUTH_DOMAIN='gmail.com',
            APPLICATION_ID='_')

        self.testbed.activate()
        self.testbed.init_memcache_stub()

        policy = datastore_stub_util.PseudoRandomHRConsistencyPolicy(probability=1)
        self.testbed.init_datastore_v3_stub(consistency_policy=policy)
        self.testbed.init_taskqueue_stub(
            root_path=os.path.abspath(os.getcwd()))
        self.testbed.init_blobstore_stub()

        try:
            self.testbed.init_images_stub()
        except:
            logging.info("PIL couldn't be loaded so you can't test with the images service.")

        self.testbed.init_logservice_stub()
        self.testbed.init_mail_stub()
        self.testbed.init_urlfetch_stub()
        self.testbed.init_user_stub()
        self.taskqueue_stub = self.testbed.get_stub(testbed.TASKQUEUE_SERVICE_NAME)

        # Search, file, and blob stubs
        stub = SearchServiceStub()
        self.testbed._register_stub('search', stub)

        blob_storage = file_blob_storage.FileBlobStorage(
            tempfile.gettempdir(),
            testbed.DEFAULT_APP_ID)
        blob_stub = blobstore_stub.BlobstoreServiceStub(blob_storage)
        file_stub = file_service_stub.FileServiceStub(blob_storage)
        self.testbed._register_stub('blobstore', blob_stub)
        self.testbed._register_stub('file', file_stub)

    def deactivate(self):
        try:
            self.testbed.deactivate()
        except:
            pass

    def run_deferred_tasks(self, queue='default'):
        tasks = self.taskqueue_stub.GetTasks(queue)
        while tasks:
            self.taskqueue_stub.FlushQueue(queue)
            for task in tasks:
                deferred.run(base64.b64decode(task['body']))
            tasks = self.taskqueue_stub.GetTasks(queue)

    def login_user(self, email, admin=False):
        self.testbed.setup_env(
            USER_EMAIL=email,
            USER_ID=email,
            USER_IS_ADMIN='1' if admin else '0',
            AUTH_DOMAIN='gmail.com',
            overwrite=True)



from setuptools import setup

VERSION = '2.1.5'

setup(
    name="FerrisNose",
    version=VERSION,
    author='Jon Parrott',
    author_email='jjramone13@gmail.com',
    maintainer='Jon Parrott',
    maintainer_email='jjramone13@gmail.com',
    description='Nose plugin for testing Google App Engine application. Designed for the Ferris Framework but should work for any App Engine app.',
    url='https://bitbucket.org/jonparrott/ferrisnose',
    license='Apache License 2.0',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
    ],
    entry_points={
        'nose.plugins.0.10': [
            'ferris = ferrisnose.plugin:FerrisNose'
        ]
    },
    packages=['ferrisnose'],
    install_requires=[
        'nose>=1.3.0',
        'google-api-python-client>=1.2.0',
        'oauth2client>=1.2.0',
        'webtest',
        'protobuf>=2.4.0',
        'protorpc>=0.9.0',
    ],
)



import unittest
import webtest


class AppEngineTest(unittest.TestCase):
    """
    Basic class from which all app engine test cases can inherit from.
    Handles setting up the testbed and provides utilities to log in
    users and run deferred tasks.
    """

    def setUp(self):
        from .testbed import FullTestBed
        self.testbed = FullTestBed()
        self.testbed.activate()

    def tearDown(self):
        self.testbed.deactivate()

    def loginUser(self, email='test@example.com', admin=False):
        self.testbed.login_user(email, admin)

    login_user = loginUser

    def runDeferredTasks(self, queue='default'):
        self.testbed.run_deferred_tasks(queue)

    run_deferred_tasks = runDeferredTasks


class AppEngineWebTest(AppEngineTest):
    """
    Provides a complete app engine testbed as well as a webtest instance
    available at ``self.testapp``. You can add routes using ``self.add_route``
    or add a ferris controller using ``self.add_controller``.
    """
    def setUp(self):
        from webapp2 import WSGIApplication
        super(AppEngineWebTest, self).setUp()
        app = WSGIApplication(debug=True, config={
            'webapp2_extras.sessions': {'secret_key': 'notasecret'}
        })
        self.testapp = webtest.TestApp(app)

    def add_controller(self, c):
        c._build_routes(self.testapp.app.router)

    addController = add_controller

    def add_route(self, r):
        self.testapp.app.router.add(r)

    addRoute = add_route


class FerrisAppTest(AppEngineTest):
    """
    Provides a complete App Engine test environment and also automatically routes
    all application and plugin handlers to ``testapp``.
    """
    def setUp(self):
        super(FerrisAppTest, self).setUp()

        import main
        reload(main)
        self.testapp = webtest.TestApp(main.main_app)
